{************************************************}
{                                                }
{   Application Editor 1.0                       }
{   ViewText Unit                                }
{   Copyright (c) 1995 by Dialektika Ltd.        }
{                                                }
{************************************************}
{&Use32+}
unit ViewText;

{$X+,S-}

interface

uses
  Objects, Drivers, Views;

const
  cmTransData   = 33000;

type
  PLineCollection = ^TLineCollection;
  TLineCollection = object(TCollection)
    procedure FreeItem(Item: Pointer); virtual;
    function GetItem(var S: TStream): Pointer; virtual;
    procedure PutItem(var S: TStream; Item: Pointer); virtual;
  end;

  PFileViewer = ^TFileViewer;
  TFileViewer = object(TScroller)
    FileName: FNameStr;
    FileLines: PCollection;
    IsValid: Boolean;
    constructor Init(var Bounds: TRect; AHScrollBar, AVScrollBar: PScrollBar;
      const AFileName: FNameStr);
    destructor Done; virtual;
    constructor Load(var S: TStream);
    procedure Store(var S: TStream);
    procedure Draw; virtual;
    procedure ReadFile;
    procedure SetState(AState: Word; Enable: Boolean); virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
  end;

  PTextWindow = ^TTextWindow;
  TTextWindow = object(TWindow)
    constructor Init(R: TRect; const FileName: FNameStr; ANumber: Integer);
    function GetTitle(MaxSize: Integer): TTitleStr; virtual;
  end;

const
  RLineCollection: TStreamRec = (
     ObjType: 59962;
     VmtLink: Ofs(TypeOf(TLineCollection)^);
     Load:    @TLineCollection.Load;
     Store:   @TLineCollection.Store
  );

  RFileViewer: TStreamRec = (
     ObjType: 59997;
     VmtLink: Ofs(TypeOf(TFileViewer)^);
     Load:    @TFileViewer.Load;
     Store:   @TFileViewer.Store
  );

  RTextWindow: TStreamRec = (
     ObjType: 59996;
     VmtLink: Ofs(TypeOf(TTextWindow)^);
     Load:    @TTextWindow.Load;
     Store:   @TTextWindow.Store
  );

procedure RegisterViewText;

implementation

uses
  Dos, Memory, App, MsgBox;

procedure TLineCollection.FreeItem(Item: Pointer);
begin
  DisposeStr(Item)
end;

function TLineCollection.GetItem(var S: TStream): Pointer;
var
  P: Pointer;
begin
  GetItem := S.ReadStr
end;

procedure TLineCollection.PutItem(var S: TStream; Item: Pointer);
begin
  S.WriteStr(Item)
end;

constructor TFileViewer.Init(var Bounds: TRect; AHScrollBar,
  AVScrollBar: PScrollBar; const AFileName: FNameStr);
begin
  inherited Init(Bounds, AHScrollBar, AVScrollBar);
  GrowMode := gfGrowHiX + gfGrowHiY;
  if AFileName <> '' then
  begin
    FileName := FExpand(AFileName);
    ReadFile
  end
end;

destructor TFileViewer.Done;
begin
  if Assigned(FileLines) then Dispose(FileLines, Done);
  inherited Done
end;

constructor TFileViewer.Load(var S: TStream);
begin
  inherited Load(S);
  S.Read(FileName[0], SizeOf(Char));
  S.Read(FileName[1], Length(FileName));
  FileLines := nil;
  ReadFile
end;

procedure TFileViewer.Store(var S: TStream);
begin
  inherited Store(S);
  S.Write(FileName, Length(FileName) + 1);
end;

procedure TFileViewer.Draw;
var
  B: TDrawBuffer;
  C: Byte;
  I: Integer;
  S: String;
  P: PString;
begin
  C := GetColor(1);
  for I := 0 to Size.Y - 1 do
  begin
    MoveChar(B, ' ', C, Size.X);
    if Assigned(FileLines) then
    if Delta.Y + I < FileLines^.Count then
    begin
      P := FileLines^.At(Delta.Y + I);
      if P <> nil then S := Copy(P^, Delta.X + 1, Size.X)
      else S := '';
      MoveStr(B, S, C);
    end;
    WriteLine(0, I, Size.X, 1, B);
  end;
end;

procedure TFileViewer.ReadFile;
var
  FileToView: Text;
  Line: String;
  MaxWidth: Integer;
  E: TEvent;
begin
  if FileName = '' then Exit;
  IsValid := True;
  if Assigned(FileLines) then Dispose(FileLines, Done);
  FileLines := New(PLineCollection, Init(5, 5));
  {$I-}
  Assign(FileToView, FileName);
  Reset(FileToView);
  if IOResult <> 0 then
  begin
    if Assigned(Application) then
      MessageBox('Cannot open file ' + FileName + '.', nil, mfError + mfOkButton);
    IsValid := False;
  end
  else
  begin
    MaxWidth := 0;
    while not Eof(FileToView) and not LowMemory do
    begin
      Readln(FileToView, Line);
      if Length(Line) > MaxWidth then MaxWidth := Length(Line);
      FileLines^.Insert(NewStr(Line));
    end;
    Close(FileToView);
  end;
  {$I+}
  Limit.X := MaxWidth;
  Limit.Y := FileLines^.Count;
end;

procedure TFileViewer.SetState(AState: Word; Enable: Boolean);
begin
  inherited SetState(AState, Enable);
  if Enable and (AState and sfExposed <> 0) then
     SetLimit(Limit.X, Limit.Y);
end;

procedure TFileViewer.HandleEvent(var Event: TEvent);

procedure TargetData;
var
  AFileName: FNameStr;
begin
  if Assigned(Event.InfoPtr) then
  begin
    AFileName := String(Event.InfoPtr^);
    if AFileName = '' then FileName := ''
    else FileName := FExpand(AFileName);
    ReadFile;
    DrawView;
  end;
end;

begin
  inherited HandleEvent(Event);
  case Event.What of
    evCommand:
      case Event.Command of
        cmTransData: TargetData;
      else
        Exit;
      end;
  else
    Exit;
  end;
  ClearEvent(Event);
end;

constructor TTextWindow.Init(R: TRect; const FileName: FNameStr; ANumber: Integer);
begin
  inherited Init(R, Filename, ANumber);
  Options := Options or ofTileable;
  GetExtent(R);
  R.Grow(-1, -1);
  Insert(New(PFileViewer, Init(R,
    StandardScrollBar(sbHorizontal + sbHandleKeyboard),
    StandardScrollBar(sbVertical + sbHandleKeyboard), FileName)));
end;

function TTextWindow.GetTitle(MaxSize: Integer): TTitleStr;
var
  P: PObject;
begin
  PObject(P) := Current;
  if Assigned(P) and (TypeOf(P^) = TypeOf(TFileViewer)) then
  with PFileViewer(P)^ do
  begin
    Flags := 0;
    GetTitle := '[FidoNetPolicyDocument]'
  end
  else inherited GetTitle(MaxSize)
end;

procedure RegisterViewText;
begin
  RegisterType(RFileViewer);
  RegisterType(RTextWindow);
end;

end.
